@@ -1,8 +1,10 @@
 import os
+import time
 import keras_cv_attention_models
 import tensorflow as tf
 from tensorflow import keras
 from keras_cv_attention_models.imagenet import callbacks, losses
+from keras_cv_attention_models import model_surgery
 
 GLOBAL_STRATEGY = None
 
@@ -21,7 +23,7 @@ def init_global_strategy(enable_float16=True, seed=0, TPU=False):
         tf.config.experimental_connect_to_cluster(resolver)
         # This is the TPU initialization code that has to be at the beginning.
         tf.tpu.experimental.initialize_tpu_system(resolver)
-        print("All devices: ", tf.config.list_logical_devices("TPU"))
+        print("[TPU] All devices: ", tf.config.list_logical_devices("TPU"))
         strategy = tf.distribute.TPUStrategy(resolver)
     elif len(gpus) > 1:
         strategy = tf.distribute.MirroredStrategy()
@@ -97,25 +99,43 @@ def init_loss(bce_threshold=1.0, label_smoothing=0):
     return loss
 
 
-def init_model(model, input_shape=(224, 224, 3), num_classes=1000, pretrained=None, restore_path=None, **kwargs):
+def init_model(model=None, input_shape=(224, 224, 3), num_classes=1000, pretrained=None, reload_compile=True, **kwargs):
+    """model Could be:
+    1. Saved h5 model path.
+    2. Model name defined in this repo, format [sub_dir].[model_name] like regnet.RegNetZD8.
+    3. timm model like timm.models.resmlp_12_224
+    """
     if isinstance(model, keras.models.Model):
         print(">>> Got a keras.models.Model: {}, do nothing with it.".format(model.name))
         return model
 
-    print(">>>> init_model kwargs:", kwargs)
-    model_name = model.strip().split(".")
-    if restore_path:
+    if model.startswith("timm."):  # model like: timm.models.resmlp_12_224
+        import timm
+        from keras_cv_attention_models.imagenet.eval_func import TorchModelInterf
+
+        print(">>>> Timm model provided:", model)
+        timm_model_name = ".".join(model.split(".")[2:])
+        model = getattr(timm.models, timm_model_name)(pretrained=True, img_size=input_shape[:2], num_classes=num_classes)
+        return TorchModelInterf(model)
+
+    if model.endswith(".h5"):
         import tensorflow_addons as tfa
 
-        print(">>>> Restore model from:", restore_path)
-        model = keras.models.load_model(restore_path)
+        print(">>>> Restore model from:", model)
+        model = keras.models.load_model(model, compile=reload_compile)
+        return model
+
+    if input_shape != -1:
+        kwargs.update({"input_shape": input_shape})  # Use model default input_shape if not specified
+    print(">>>> init_model kwargs:", kwargs)
+
+    model_name = model.strip().split(".")
+    if len(model_name) == 1:
+        model = getattr(keras.applications, model_name[0])(classes=num_classes, weights=pretrained, **kwargs)
     else:
-        if len(model_name) == 1:
-            model = getattr(keras.applications, model_name[0])(classes=num_classes, weights=pretrained, input_shape=input_shape, **kwargs)
-        else:
-            model_class = getattr(getattr(keras_cv_attention_models, model_name[0]), model_name[1])
-            model = model_class(num_classes=num_classes, input_shape=input_shape, pretrained=pretrained, **kwargs)
-        print(">>>> Built model name:", model.name)
+        model_class = getattr(getattr(keras_cv_attention_models, model_name[0]), model_name[1])
+        model = model_class(num_classes=num_classes, pretrained=pretrained, **kwargs)
+    print(">>>> Built model name:", model.name)
 
     if model_name[0] == "aotnet" and pretrained is not None and pretrained.endswith(".h5"):
         # Currently aotnet not loading from pretrained...
@@ -124,24 +144,36 @@ def init_model(model, input_shape=(224, 224, 3), num_classes=1000, pretrained=No
     return model
 
 
-def compile_model(model, optimizer, lr_base, weight_decay, bce_threshold=1.0, label_smoothing=0, loss=None, metrics=["acc"]):
+def model_post_process(model, freeze_backbone=False, freeze_norm_layers=False, use_token_label=False):
+    if freeze_backbone:
+        pool_layer_id = model_surgery.get_global_avg_pool_layer_id(model)
+        for id in range(pool_layer_id):
+            model.layers[id].trainable = False
+
+    if freeze_norm_layers:
+        for ii in model.layers:
+            if isinstance(ii, keras.layers.BatchNormalization) or isinstance(ii, keras.layers.LayerNormalization):
+                ii.trainable = False
+
+    if use_token_label:
+        model = model_surgery.convert_to_token_label_model(model)
+    return model
+
+
+def compile_model(model, optimizer, lr_base, weight_decay, loss, loss_weights=None, metrics=["acc"]):
     if isinstance(optimizer, str):
         optimizer = optimizer.lower()
         if optimizer == "sgd" and weight_decay > 0:
             # Add L2 regularizer
-            from keras_cv_attention_models import model_surgery
-
             model = model_surgery.add_l2_regularizer_2_model(model, weight_decay=weight_decay, apply_to_batch_normal=False)
         optimizer = init_optimizer(optimizer, lr_base, weight_decay)
-    if loss is None:
-        loss = init_loss(bce_threshold, label_smoothing)
     print(">>>> Loss: {}, Optimizer: {}".format(loss.__class__.__name__, optimizer.__class__.__name__))
-    model.compile(optimizer=optimizer, loss=loss, metrics=metrics)
+    model.compile(optimizer=optimizer, loss=loss, loss_weights=loss_weights, metrics=metrics)
     return model
 
 
 # @tf.function(jit_compile=True)
-def train(compiled_model, epochs, train_dataset, test_dataset=None, initial_epoch=0, lr_scheduler=None, basic_save_name=None, init_callbacks=[]):
+def train(compiled_model, epochs, train_dataset, test_dataset=None, initial_epoch=0, lr_scheduler=None, basic_save_name=None, init_callbacks=[], logs="auto"):
     if compiled_model.compiled_loss is None:
         print(">>>> Error: Model NOT compiled.")
         return None
@@ -164,6 +196,11 @@ def train(compiled_model, epochs, train_dataset, test_dataset=None, initial_epoc
         os.rename(hist_file, hist_file + ".bak")
     cur_callbacks.append(callbacks.MyHistory(initial_file=hist_file))
     cur_callbacks.append(keras.callbacks.TerminateOnNaN())
+    if logs is not None:
+        logs = "logs/" + basic_save_name + "_" + time.strftime("%Y%m%d-%H%M%S") if logs == "auto" else logs
+        cur_callbacks.append(keras.callbacks.TensorBoard(log_dir=logs, histogram_freq=1))
+        print(">>>> TensorBoard log path:", logs)
+
     if lr_scheduler is not None:
         cur_callbacks.append(lr_scheduler)
 
@@ -184,4 +221,7 @@ def train(compiled_model, epochs, train_dataset, test_dataset=None, initial_epoc
         use_multiprocessing=True,
         workers=8,
     )
+
+    if logs is not None:
+        print(">>>> TensorBoard log path:", logs)
     return checkpoint_callback.latest_save, hist
